﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Globalization;
using System.Net;
using System.IO;

namespace LightningOverlay
{
    public partial class frmLightningOverlay : Form
    {
        ManagedProj4Projection _proj = new ManagedProj4Projection();

        public frmLightningOverlay()
        {
            InitializeComponent();
        }

        private void frmLightningOverlay_Load(object sender, EventArgs e)
        {
            LoadMapData(54004);
        }

        private void LoadMapData(int proj)
        {
            try
            {
                LoadDefaultMap(proj);
                LightningOverlay();

                //wfMap.CurrentExtent = _proj.ConvertToExternalProjection(new RectangleShape(-180, 90, 180, -90));
                wfMap.Refresh();
            }
            catch (Exception err)
            {
                MessageBox.Show(err.Message);
            }
        }

        private void LoadDefaultMap(int proj)
        {
            try
            {
                if (wfMap.Overlays.Count > 0)
                    wfMap.Overlays.Clear();
                if (wfMap.AdornmentOverlay.Layers.Count > 0)
                    wfMap.AdornmentOverlay.Layers.Clear();

                _proj.InternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(4326);
                if(proj==54004)
                    _proj.ExternalProjectionParametersString = ManagedProj4Projection.GetEsriParametersString(54004);
                if(proj==4326)
                    _proj.ExternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(4326);

                if (!_proj.IsOpen)
                    _proj.Open();

                wfMap.MapUnit = _proj.GetExternalGeographyUnit();
                wfMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

                GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
                graticuleAdornmentLayer.Projection = _proj;
                graticuleAdornmentLayer.GraticuleLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.SimpleColors.Silver, 0.001F, LineDashStyle.Dash, false);
                wfMap.AdornmentOverlay.Layers.Add("GraticuleAdornmentLayer", graticuleAdornmentLayer);

                ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(Application.StartupPath + @"\Data\Countries02.shp");
                worldLayer.FeatureSource.Projection = _proj;
                worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
                worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                LayerOverlay overlay_WorldLayer = new LayerOverlay();
                overlay_WorldLayer.Layers.Add("WorldLayer", worldLayer);
                wfMap.Overlays.Add("WorldOverlay", overlay_WorldLayer);

                worldLayer.Open();
                wfMap.CurrentExtent = worldLayer.GetBoundingBox();
                worldLayer.Close();
            }
            catch (Exception err)
            {
                MessageBox.Show(err.Message);
            }
        }

        private void LightningOverlay()
        {
            try
            {
                GdiPlusRasterLayer lightningRadar = new GdiPlusRasterLayer(Application.StartupPath + @"\Data\L_plot_global_map.jpg", new RectangleShape(new PointShape(-211.714, 107.598), new PointShape(186.271, -112.4894)));
                lightningRadar.Transparency = 190;
                lightningRadar.ImageSource.Projection = _proj;

                if (wfMap.Overlays.Contains("LightningOverlay"))
                    wfMap.Overlays.Remove("LightningOverlay");

                LayerOverlay ol_Lightning = new LayerOverlay();
                ol_Lightning.Layers.Add("Lightninglayer", lightningRadar);
                wfMap.Overlays.Add("LightningOverlay", ol_Lightning);

                lightningRadar.Open();
                wfMap.CurrentExtent = lightningRadar.GetBoundingBox();
                lightningRadar.Close();

                wfMap.Refresh(ol_Lightning);
            }
            catch (Exception err)
            {
                MessageBox.Show(err.Message);
            }
        }

        Vertex v;
        private void wfMap_MouseMove(object sender, MouseEventArgs e)
        {
            PointShape WorldPointR = ExtentHelper.ToWorldCoordinate(wfMap.CurrentExtent, e.X, e.Y, wfMap.Width, wfMap.Height);
            if (!string.IsNullOrEmpty(_proj.ExternalProjectionParametersString))
                v = _proj.ConvertToInternalProjection(Convert.ToDouble(WorldPointR.X), Convert.ToDouble(WorldPointR.Y));
            else
                v = new Vertex(Convert.ToDouble(WorldPointR.X), Convert.ToDouble(WorldPointR.Y));
            statusStrip1.Items[1].Text = string.Format(CultureInfo.InvariantCulture, "Long: {0:F4} Lat: {1:F4}", v.X, v.Y);

            if ((v.X > -180 && v.X < 180) && (v.Y > -90 && v.Y < 90))
            {
                statusStrip1.Items[0].Text = DecimalDegreesHelper.GetDegreesMinutesSecondsStringFromDecimalDegreePoint(new PointShape(v));
            }
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton1.Checked)
            {
                LoadMapData(4326);
            }
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton2.Checked)
            {
                LoadMapData(54004);
            }
        }

    }
}
